@@ -32,7 +32,7 @@ class Abstract(object):
       return self._parse_response(response)
     except requests.exceptions.RequestException as error:
       eprint('ERROR: {}'.format(error))
-      return { 'error': True, 'msg': error}
+      return {'error': True, 'msg': error}
 
   def get(self, obj_id, cache=False, params={}, verify=True): # pylint: disable=dangerous-default-value
     '''Get detail of an resource'''
@@ -44,7 +44,7 @@ class Abstract(object):
     params.update(self.payload)
     try:
       response = requests.get(
-        '{}{}'.format(self.endpoint, obj_id),
+        self.resource_url(obj_id),
         params=params,
         verify=verify,
       )
@@ -91,12 +91,16 @@ class Abstract(object):
       offset += limit
     return item_list
 
+  def resource_url(self, obj_id):
+    '''Generate resource uri from obj id'''
+    return '{}{}'.format(self.endpoint, obj_id)
+
   def save(self, obj_id=None, params={}, payload={}, verify=True): # pylint: disable=dangerous-default-value
     '''Save or update resource'''
     params.update(self.payload)
     try:
       response = getattr(requests, 'put' if obj_id else 'post')(
-        '{}{}'.format(self.endpoint, obj_id or ''),
+        self.resource_url(obj_id) if obj_id else self.endpoint,
         data=json.dumps(payload),
         headers=self.headers,
         params=params,
@@ -120,8 +124,23 @@ class Abstract(object):
       eprint('ERROR: {}'.format(error_msgs[response.status_code]))
       return {'error': True, 'msg': error_msgs[response.status_code]}
 
+    if response.status_code == 204:  # for delete response
+      return {'error': False}
+
     try:
-      return response.json()
+      response = response.json()
+
+      error = response and response.get('error')
+      if error and isinstance(error, dict):
+        response = error
+
+        if response.get('error_msgs'):
+          eprint('ERROR: {}'.format(response['error_msgs']))
+
+        if response.get('warning_msgs'):
+          eprint('WARNING: {}'.format(response['warning_msgs']))
+
+      return response
     except json.decoder.JSONDecodeError as error:
       msg = 'ERROR: Not able to parse response: {}.'.format(error)
       eprint(msg)
