@@ -6,5 +6,5 @@ from .abstract import Abstract
 class Module(Abstract):
   '''Webapp Module class'''
   def __init__(self, cfg):
-    super().__init__(cfg)
+    super(Module, self).__init__(cfg)
     self.endpoint += 'modules/'
