@@ -6,5 +6,5 @@ from .abstract import Abstract
 class Upload(Abstract):
   '''Webapp Upload class'''
   def __init__(self, cfg):
-    super().__init__(cfg)
+    super(Upload, self).__init__(cfg)
     self.endpoint += 'uploads/'
